/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ex: set tabstop=8 softtabstop=4 shiftwidth=4 expandtab: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla PostScript driver printer list component.
 *
 * The Initial Developer of the Original Code is
 * Kenneth Herron <kherron+mozilla@fmailbox.com>
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Copyright 2004 Sun Microsystems, Inc.  All Rights Reserved.
 * Use is subject to license terms.
 * Email to: Norm.Jacobs@Sun.COM
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsDebug.h"
#include "nsPAPIShim.h"
#include "prlink.h"

// List of symbols to find in libcups. Must match symAddr[] defined in Init().
// Making this an array of arrays instead of pointers allows storing the
// whole thing in read-only memory.
static const char gSymName[][sizeof("papiPrinterGetAttributeList")] = {
    { "papiServiceCreate" },
    { "papiServiceDestroy" },
    { "papiPrintersList" },
    { "papiPrinterQuery" },
    { "papiPrinterGetAttributeList" },
    { "papiPrinterListFree" },
    { "papiJobSubmit" },
    { "papiJobFree" },
    { "papiAttributeListGetString" },
    { "papiAttributeListAddString" },
    { "papiAttributeListAddInteger" },
    { "papiAttributeListFree" }
};
static const int gSymNameCt = sizeof(gSymName) / sizeof(gSymName[0]);


PRBool
nsPAPIShim::Init()
{
    mPAPILib = PR_LoadLibrary("libpapi.so");
    if (!mPAPILib)
        return PR_FALSE;

    // List of symbol pointers. Must match gSymName[] defined above.
    void **symAddr[] = {
        (void **)&mPapiServiceCreate,
        (void **)&mPapiServiceDestroy,
        (void **)&mPapiPrintersList,
        (void **)&mPapiPrinterQuery,
        (void **)&mPapiPrinterGetAttributeList,
        (void **)&mPapiPrinterListFree,
	(void **)&mPapiJobSubmit,
	(void **)&mPapiJobFree,
        (void **)&mPapiAttributeListGetString,
        (void **)&mPapiAttributeListAddString,
        (void **)&mPapiAttributeListAddInteger,
        (void **)&mPapiAttributeListFree,
	NULL
    };

    for (int i = gSymNameCt; i--; ) {
        *(symAddr[i]) = PR_FindSymbol(mPAPILib, gSymName[i]);
        if (! *(symAddr[i])) {
            PR_UnloadLibrary(mPAPILib);
            mPAPILib = nsnull;
            return PR_FALSE;
        }
    }
    return PR_TRUE;
}

nsPAPIShim::~nsPAPIShim()
{
    if (mPAPILib)
        PR_UnloadLibrary(mPAPILib);
}

static void
AddPAPIPrinter(nsCStringArray& aList, char *name)
{
	if ((name != NULL) && (strcmp(name, "_default") != 0)) {
		nsCAutoString fullName(NS_PAPI_PRINTER);
		fullName.Append(name);
		aList.AppendCString(fullName);
	}
}

void
nsPAPIShim::PopulatePrinterList(nsCStringArray& aList)
{
        papi_status_t status;
        papi_service_t svc = NULL;
        papi_printer_t *list = NULL,
                        printer = NULL;
	int count;
        char *interest_attrs[] = { "member-names", NULL };
        char *attributes[] = { "printer-name", NULL };

        // In case the "dynamic" papi library is in use, it is
        // user, system, and name service aware.  This means that
        // "all" printers may be extraordinarily large.  we should
        // look for an "_all" pseudo printer entry, this entry will
	// contain a list of "interesting" printers.  The list may
	// be defined at any level and will be located based on 
	// configurable lookup ordering.
        if (mPapiServiceCreate(&svc, NULL, NULL, NULL, NULL,
                        PAPI_ENCRYPT_NEVER, NULL) != 0)
                return;

	count = aList.Count();
        status = mPapiPrinterQuery(svc, "_all", interest_attrs, NULL, &printer);
        if ((status == 0) && (printer != NULL)) {
                char *name = NULL;
                void *iter = NULL;

                papi_attribute_t **attrs = mPapiPrinterGetAttributeList(printer);
                for (status = mPapiAttributeListGetString(attrs, &iter,
                                        	"member-names", &name);
                     status == 0;
                     status = mPapiAttributeListGetString(attrs, &iter,
                                        	NULL, &name))
			AddPAPIPrinter(aList, name);
        }

	// If trying "_all" printers didn't add any entries to the list,
	// enumerate all of the printers
	if (aList.Count() == count)
		status = mPapiPrintersList(svc, attributes, NULL, &list);
	if ((status == 0) && (list != NULL)) {
                int i = 0;

                for (i = 0; list[i] != NULL; i++) {
                        char *name = NULL;
                        void *iter = NULL;
                        papi_attribute_t **attrs =
                                mPapiPrinterGetAttributeList(list[i]);

                        status = mPapiAttributeListGetString(attrs, &iter,
                                        	"printer-name", &name);
                        if (status == 0)
				AddPAPIPrinter(aList, name);
                }
        }

        mPapiServiceDestroy(svc);
}

int
nsPAPIShim::PrintFile(char *printer, char *file, int copies, char *title)
{
	int result = 0;
        papi_status_t status;
        papi_service_t svc = NULL;
        papi_printer_t job = NULL;
        papi_attribute_t **attributes = NULL;
        char *files[] = { NULL, NULL };

        status = mPapiServiceCreate(&svc, NULL, NULL, NULL, NULL,
                        PAPI_ENCRYPT_NEVER, NULL);
        if (status != 0)
                return -1;

	if (copies < 1)
		copies = 1;

	mPapiAttributeListAddString(&attributes, PAPI_ATTR_EXCL,
			"job-name", title);
	mPapiAttributeListAddInteger(&attributes, PAPI_ATTR_EXCL,
			"copies", copies);
	files[0] = file;
	status = mPapiJobSubmit(svc, printer, attributes, NULL, files, &job);
	mPapiJobFree(job);

	if (status != 0)
		result = -1;
		
	mPapiServiceDestroy(svc);

	return (result);
}
